@@ -11,107 +11,113 @@ import unittest
 
 import ezdxf
 
+DWG = ezdxf.new('AC1009')
+
+
+def new_drawing():
+    dwg = ezdxf.new('AC1009')
+    msp = dwg.modelspace()
+    return dwg, msp
+
 
 class SetupDrawing(unittest.TestCase):
     def setUp(self):
-        self.dwg = ezdxf.new('AC1009')
-        self.layout = self.dwg.modelspace()
+        self.layout = DWG.modelspace()
 
 
 class TestModelSpace(SetupDrawing):
-    def test_layout_property(self):
-        line = self.layout.add_line((0, 0), (1, 1))
-        self.assertEqual(self.layout, line.layout)
-
     def test_drawing_attribute(self):
         line = self.layout.add_line((0, 0), (1, 1))
-        self.assertEqual(self.dwg, line.drawing)
+        self.assertEqual(DWG, line.drawing)
 
     def test_dxffactory_property(self):
         line = self.layout.add_line((0, 0), (1, 1))
-        self.assertEqual(self.dwg.dxffactory, line.dxffactory)
-
-    def test_layout_attribute(self):
-        line = self.layout.add_line((0, 0), (1, 1))
-        self.assertEqual(self.layout, line.layout)
+        self.assertEqual(DWG.dxffactory, line.dxffactory)
 
     def test_delete_entity(self):
+        dwg, layout = new_drawing()
         for _ in range(5):
-            self.layout.add_line((0, 0), (10, 0))
-        lines = self.layout.query('LINE')
+            layout.add_line((0, 0), (10, 0))
+        lines = layout.query('LINE')
         self.assertEqual(5, len(lines))
         line3 = lines[2]
-        self.layout.delete_entity(line3)
+        layout.delete_entity(line3)
         self.assertTrue(line3.dxf.paperspace < 0, "Paper space attribute has to be invalid (<0).")
-        self.assertFalse(line3 in self.layout)
-        self.assertFalse(line3.dxf.handle in self.dwg.entitydb)
+        self.assertFalse(line3 in layout)
+        self.assertFalse(line3.dxf.handle in dwg.entitydb)
 
     def test_delete_polyline(self):
+        entity_count = len(list(self.layout))
         pline = self.layout.add_polyline3d([(0, 0, 0), (1, 2, 3), (4, 5, 6)])
-        self.assertEqual(5, len(list(self.layout)))  # 1x POLYLINE + 3x VERTEX + 1x SEQEND
+        self.assertEqual(entity_count+1, len(list(self.layout)))  # 1x POLYLINE rest is linked to the POLYLINE entity
         self.layout.delete_entity(pline)
-        self.assertEqual(0, len(list(self.layout)))
+        self.assertEqual(entity_count, len(list(self.layout)))
 
     def test_delete_blockref_with_attribs(self):
+        entity_count = len(list(self.layout))
         blockref = self.layout.add_blockref("TESTBLOCK", (0, 0))
         blockref.add_attrib('TAG1', "Text1", (0, 1))
         blockref.add_attrib('TAG2', "Text2", (0, 2))
         blockref.add_attrib('TAG3', "Text3", (0, 3))
-        self.assertEqual(5, len(list(self.layout)))  # 1x INSERT + 3x ATTRIB + 1x SEQEND
+        self.assertEqual(entity_count+1, len(list(self.layout)))  # 1x INSERT, rest is linked to the INSERT entity
         self.layout.delete_entity(blockref)
-        self.assertEqual(0, len(list(self.layout)))
+        self.assertEqual(entity_count, len(list(self.layout)))
 
     def test_delete_all_entities(self):
-        paperspace = self.dwg.layout()
-        modelspace = self.dwg.modelspace()
+        paper_space = DWG.layout()
+        paper_space_count = len(paper_space)
+        model_space = DWG.modelspace()
+        model_space_count = len(model_space)
         for _ in range(5):
-            modelspace.add_line((0, 0), (1, 1))
-            paperspace.add_line((0, 0), (1, 1))
+            model_space.add_line((0, 0), (1, 1))
+            paper_space.add_line((0, 0), (1, 1))
 
-        self.assertEqual(5, len(list(modelspace)))
-        self.assertEqual(5, len(list(paperspace)))
+        self.assertEqual(model_space_count + 5, len(model_space))
+        self.assertEqual(paper_space_count + 5, len(paper_space))
 
-        modelspace.delete_all_entities()
-        self.assertEqual(0, len(list(modelspace)))
-        self.assertEqual(5, len(list(paperspace)))
+        model_space.delete_all_entities()
+        self.assertEqual(0, len(model_space))
+        self.assertEqual(paper_space_count + 5, len(paper_space))
 
 
 class TestPaperSpace(SetupDrawing):
     def test_paper_space(self):
-        paperspace = self.dwg.layout('Name it like you want, there is only one paperspace at AC1009')
+        paperspace = DWG.layout('Name it like you want, there is only one paperspace at AC1009')
         line = paperspace.add_line((0, 0), (1, 1))
         self.assertEqual(1, line.dxf.paperspace)
-        self.assertEqual(paperspace, line.layout)
 
     def test_iter_layout(self):
-        self.layout.add_line((0, 0), (1, 1))
-        self.layout.add_line((0, 0), (1, 1))
-        entities = list(self.layout)
+        paper_space = DWG.layout()
+        paper_space.delete_all_entities()
+        paper_space.add_line((0, 0), (1, 1))
+        paper_space.add_line((0, 0), (1, 1))
+        entities = list(paper_space)
         self.assertEqual(2, len(entities))
         # Are all necessary attribute set?
         e = entities[0]
-        self.assertEqual(self.dwg, e.drawing)
-        self.assertEqual(self.layout, e.layout)
+        self.assertEqual(DWG, e.drawing)
 
     def test_query_entities(self):
-        self.layout.add_line((0, 0), (1, 1), dxfattribs={'layer': 'lay_lines'})
-        self.layout.add_line((0, 0), (1, 1), dxfattribs={'layer': 'lay_lines'})
-        entities = self.layout.query('*[layer ? "lay_.*"]')
+        paper_space = DWG.layout()
+        paper_space.delete_all_entities()
+        paper_space.add_line((0, 0), (1, 1), dxfattribs={'layer': 'lay_lines'})
+        paper_space.add_line((0, 0), (1, 1), dxfattribs={'layer': 'lay_lines'})
+        entities = paper_space.query('*[layer ? "lay_.*"]')
         self.assertEqual(2, len(entities))
         # Are all necessary attribute set?
         e = entities[0]
-        self.assertEqual(self.dwg, e.drawing)
-        self.assertEqual(self.layout, e.layout)
+        self.assertEqual(DWG, e.drawing)
 
     def test_model_space_get_layout_for_entity(self):
-        line = self.layout.add_line((0, 0), (1, 1))
-        layout = self.dwg.layouts.get_layout_for_entity(line)
-        self.assertEqual(self.layout, layout)
+        model_space = DWG.modelspace()
+        line = model_space.add_line((0, 0), (1, 1))
+        layout = DWG.layouts.get_layout_for_entity(line)
+        self.assertEqual(model_space, layout)
 
     def test_paper_space_get_layout_for_entity(self):
-        paper_space = self.dwg.layout()
+        paper_space = DWG.layout()
         line = paper_space.add_line((0, 0), (1, 1))
-        layout = self.dwg.layouts.get_layout_for_entity(line)
+        layout = DWG.layouts.get_layout_for_entity(line)
         self.assertEqual(paper_space, layout)
 
 
@@ -124,7 +130,7 @@ class TestGraphicsDefaultSettings(SetupDrawing):
         self.assertEqual((0.0, 0.0, 1.0), line.dxf.extrusion)
 
 
-class TestGenericWrapper2(SetupDrawing):
+class TestDXFEntity(SetupDrawing):
     def test_clone_dxf_attribs(self):
         line = self.layout.add_line((0, 0), (1, 1))
         attribs = line.clone_dxf_attribs()
@@ -210,6 +216,22 @@ class TestText(SetupDrawing):
         text.dxf.valign = 3
         self.assertEqual('TOP_CENTER', text.get_align())
 
+    def test_get_pos_TOP_CENTER(self):
+        text = self.layout.add_text('text')
+        text.set_pos((2, 2), align="TOP_CENTER")
+        align, p1, p2 = text.get_pos()
+        self.assertEqual("TOP_CENTER", align)
+        self.assertEqual(p1, (2, 2))
+        self.assertIsNone(p2)
+
+    def test_get_pos_LEFT(self):
+        text = self.layout.add_text('text')
+        text.set_pos((2, 2))
+        align, p1, p2 = text.get_pos()
+        self.assertEqual("LEFT", align)
+        self.assertEqual(p1, (2, 2))
+        self.assertIsNone(p2)
+
 
 class TestBlock(SetupDrawing):
     def test_create_blockref(self):
@@ -228,15 +250,16 @@ class TestBlock(SetupDrawing):
         ref = self.layout.add_blockref('BLOCK', (0, 0))
         ref.add_attrib('TEST1', 'text1', (0, 0))
         ref.add_attrib('TEST2', 'text2', (0, 0))
-        attribs = [attrib.dxf.tag for attrib in ref]
+        attribs = [attrib.dxf.tag for attrib in ref.attribs()]
         self.assertEqual(['TEST1', 'TEST2'], attribs)
 
     def test_has_seqend(self):
         ref = self.layout.add_blockref('BLOCK', (0, 0))
         ref.add_attrib('TEST1', 'text1', (0, 0))
         ref.add_attrib('TEST2', 'text2', (0, 0))
-        entity = self.layout.get_entity_at_index(-1)
-        self.assertEqual('SEQEND', entity.dxftype())
+        entity = ref.get_attrib('TEST2')
+        seqend = self.layout.entitydb[entity.tags.link]
+        self.assertEqual('SEQEND', seqend.dxftype())
 
     def test_insert_place(self):
         ref = self.layout.add_blockref('BLOCK', (0, 0))
